#include "heap.h"

max_heap::max_heap(int tamanyo_maximo) {                 // constructor
  tamanyo = (tamanyo_maximo > 0) ? tamanyo_maximo : 1;
  ocupados = 0;
  dist *rv = new dist[tamanyo];              // queremos empezar con indice 1
  vector = rv - 1;                            // vector[1] es rv[0],...
}

max_heap::~max_heap(){
  delete[] (vector+1);
}

int max_heap::maximo() {
  return vector[1].distancia;
}

void max_heap::heapify(int pos) {                  // versi´n iterativa
  int hijo;
  dist tmp = vector[pos];
  while (hijo_izq(pos) <= ocupados) {
    hijo = hijo_izq(pos);
    if(hijo < ocupados && (vector[hijo+1].distancia > vector[hijo].distancia || (vector[hijo+1].distancia == vector[hijo].distancia && vector[hijo+1].destino > vector[hijo].destino)))
      hijo++;                                        // el hijo derecho

    if(vector[hijo].distancia > tmp.distancia)
      vector[pos] = vector[hijo];
    else
      break;
    pos = hijo;
  }
  vector[ pos ] = tmp;
}


void max_heap::buildheap() {
  for (int i = ocupados/2; i >= 1; i--)
    heapify(i);
}

bool max_heap::extraer_maximo(dist& elemento) {
  if (ocupados <= 0)
    return false;
  elemento = vector[1];
  vector[1] = vector[ocupados];
  ocupados--;
  heapify(1);
  return true;
}

bool max_heap::insertar(unsigned int elemento, int ini, int des) {
  if (ocupados >= tamanyo)
    return false;
  ocupados++;
  int posicion = ocupados;
  while ((posicion > 1) && (elemento > vector[padre(posicion)].distancia)) {
    vector[posicion] = vector[padre(posicion)];
    posicion = padre(posicion);
  }
  vector[posicion].distancia = elemento;
  vector[posicion].inicio = ini;
  vector[posicion].destino = des;
  return true;
}

void max_heap::meter(unsigned int elemento, int ini, int des){
  vector[1].distancia=elemento;
  vector[1].inicio = ini;
  vector[1].destino = des;
  heapify(1);
}


void max_heap::imprime(){
  for(int i=1;i<=ocupados;++i){
    std::cout << "Destino: " << vector[i].destino << " Distancia: " << vector[i].distancia << std::endl;
  }
  std::cout << "\n\n";
}



//----------------------MIN_HEAP----------------------------//






min_heap::min_heap(int tamanyo_maximo) {                 // constructor
  tamanyo = (tamanyo_maximo > 0) ? tamanyo_maximo : 1;
  ocupados = 0;
  int_dijkstra *rv = new int_dijkstra[tamanyo];              // queremos empezar con indice 1
  indice = new int[tamanyo];
  vector = rv - 1;                            // vector[1] es rv[0],...
}

min_heap::~min_heap(){
  delete[] (vector+1);
  delete [] indice;
}


void min_heap::heapify(int pos) {                  // versión iterativa
  int hijo;
  int_dijkstra tmp;
  tmp.pos = vector[pos].pos;
  tmp.indice = vector[pos].indice;
  while (hijo_izq(pos) <= ocupados) {
    hijo = hijo_izq(pos);
    if(hijo < ocupados && (vector[hijo+1].pos < vector[hijo].pos))
      hijo++;                                        // el hijo derecho

    if(vector[hijo].pos < tmp.pos){
      vector[pos].pos = vector[hijo].pos;
      vector[pos].indice = vector[hijo].indice;
      indice[vector[pos].indice] = pos;
    }
    else
      break;
    pos = hijo;
  }
  vector[ pos ].pos = tmp.pos;
  vector[ pos ].indice = tmp.indice;
  indice[vector[pos].indice] = pos;
}


void min_heap::actualizar(int valor_nuevo, int index){
  int posicion = indice[index];
  while((posicion > 1) && (valor_nuevo < vector[padre(posicion)].pos)){
    vector[posicion].pos = vector[padre(posicion)].pos;
    vector[posicion].indice = vector[padre(posicion)].indice;
    indice[vector[posicion].indice] = posicion;
    posicion = padre(posicion);
  }
  vector[posicion].pos = valor_nuevo;
  vector[posicion].indice = index;
  indice[vector[posicion].indice] = posicion;
  heapify(posicion);
}


bool min_heap::extraer_minimo(int &elemento, int &index) {
  if (ocupados <= 0)
    return false;
  elemento = vector[1].pos;
  index = vector[1].indice;
  vector[1].pos = vector[ocupados].pos;
  vector[1].indice = vector[ocupados].indice;
  indice[vector[1].indice]=1;
  ocupados--;
  heapify(1);
  return true;
}

bool min_heap::insertar(int elemento, int index) {
  if (ocupados >= tamanyo) // esta comparación puede que no haga falta
    return false;
  ocupados++;
  int posicion = ocupados;
  while ((posicion > 1) && (elemento < vector[padre(posicion)].pos)) { // puede que este while no haga falta
    vector[posicion].pos = vector[padre(posicion)].pos;
    vector[posicion].indice = vector[padre(posicion)].indice;
    indice[vector[posicion].indice] = posicion;
    posicion = padre(posicion);
  }
  vector[posicion].pos = elemento;
  vector[posicion].indice = index;
  indice[vector[posicion].indice]=posicion;
  return true;
}


void min_heap::imprime(){
  for(int i=1;i<=ocupados;++i){
    std::cout << "Distancia: " << vector[i].pos << " Indice: " << vector[i].indice << " indice_vector: " << indice[vector[i].indice] << std::endl;
  }
  std::cout << "\n\n";
}
